package com.unis.project.app.controller;

import com.unis.common.utils.ResponseWrapper;
import com.unis.project.common.vo.FileQueryVO;
import com.unis.project.common.vo.FileVO;
import com.unis.project.client.FileFeign;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author WangZL
 * @version 1.0
 * @Title FileController
 * @ProjectName 文件 FileController
 * @Description  TODO
 * @date 2019/07/22 17点26分
 */
@RestController
@Api(value = "文件 ", tags = "文件")
@Slf4j
public class FileController {

	@Autowired
	private FileFeign fileFeign;

	@ApiOperation(value = "文件-下载列表", notes = "参数：数据ID")
	@PostMapping(value = "/fileList/{sjId}")
    public ResponseWrapper getFileListBySjId(@PathVariable("sjId") String sjId){
		return fileFeign.getFileListBySjId(sjId);
    }

	@ApiOperation(value = "文件-查询列表", notes = "参数：文件号，文件名称")
	@PostMapping(value = "/fileList")
	public ResponseWrapper getFileList(@RequestBody FileQueryVO fileQueryVO){
		return fileFeign.getFileList(fileQueryVO);
	}

	@ApiOperation(value = "文件-分页查询列表", notes = "参数：文件号，文件名称")
	@PostMapping(value = "/filePage/{pageNum}/{pageSize}")
	public ResponseWrapper getFilePage(@RequestBody FileQueryVO fileQueryVO ,@PathVariable("pageNum")Integer pageNum,
									   @PathVariable("pageSize") Integer pageSize) {
		return fileFeign.getFilePage(fileQueryVO,pageNum,pageSize);
	}

	@PostMapping(value = "/file")
	@ApiOperation(value = "文件-新增", notes = "参数：文件信息对象 model")
	public ResponseWrapper insert(@RequestBody FileVO fileVO) {
		return fileFeign.insert(fileVO);
	}

	@PutMapping(value = "/file")
	@ApiOperation(value = "文件-更新", notes = "参数：文件信息对象 model")
	public ResponseWrapper update(@RequestBody FileVO fileVO) {
		return fileFeign.update(fileVO);
	}

	@DeleteMapping(value="/file/{id}")
	@ApiOperation(value = "文件-删除", notes = "参数：文件ID")
	public ResponseWrapper delete(@PathVariable("id") Integer id) {
		return fileFeign.delete(id);
	}

	@GetMapping(value="/file/{id}")
	@ApiOperation(value = "文件-查询详情", notes = "参数：文件ID")
	public ResponseWrapper getFileInfoById(@PathVariable("id") Integer id) {
		return fileFeign.getFileInfoById(id);
	}

	@PostMapping(value="/file/{id}")
	@ApiOperation(value = "文件-下发", notes = "参数：文件ID")
	public ResponseWrapper updateLowerStatus(@PathVariable("id") Integer id) {
		return fileFeign.updateLowerStatus(id);
	}


	@PutMapping(value="/fileListBySjId")
	@ApiOperation(value = "文件-更新文件信息", notes = "参数：文件列表, 数据ID")
	public ResponseWrapper updateFileList(@RequestParam("sjId") String sjId,
								  			 @RequestBody List<FileVO> fileVOList) {
		return fileFeign.updateFileList(sjId,fileVOList);
	}
}
