package com.unis.project.app.controller;

import com.unis.common.utils.ResponseWrapper;
import com.unis.project.app.service.ProjectApplicationService;
import com.unis.project.client.ProjectApplicationFeign;
import com.unis.project.common.vo.ProjectApplicationVO;
import com.unis.project.common.vo.ProjectQueryVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @author WangZL
 * @version 1.0
 * @Title ProjectApplicationController
 * @ProjectName 项目申请 ProjectApplicationController
 * @Description  TODO
 * @date 2019/07/22 17点26分
 */
@RestController
@Api(value = "项目申请 ", tags = "项目申请")
@Slf4j
public class ProjectApplicationController {

	@Autowired
	private ProjectApplicationFeign projectApplicationFeign;

	@Autowired
	private ProjectApplicationService projectApplicationService;

	@ApiOperation(value = "项目申请-查询列表", notes = "参数：项目方案标题，项目年度")
	@PostMapping(value = "/projectApplicationList")
	public ResponseWrapper getProjectApplicationList(@RequestBody ProjectQueryVO projectQueryVO){
		return projectApplicationFeign.getProjectApplicationList(projectQueryVO);
	}

	@ApiOperation(value = "项目申请-分页查询列表", notes = "参数：项目方案标题，项目年度")
	@PostMapping(value = "/projectApplicationPage/{pageNum}/{pageSize}")
	public ResponseWrapper getProjectApplicationPage(@RequestBody ProjectQueryVO projectQueryVO ,@PathVariable("pageNum")Integer pageNum,
													 @PathVariable("pageSize") Integer pageSize) {
		return projectApplicationFeign.getProjectApplicationPage(projectQueryVO,pageNum,pageSize);
	}

	@PostMapping(value = "/projectApplication/{dataStatus}")
	@ApiOperation(value = "项目申请-根据状态保存数据", notes = "参数：项目申请信息，操作状态 0、保存。1、提交")
	public ResponseWrapper projectApplication(@RequestBody ProjectApplicationVO projectApplicationVO,@PathVariable("dataStatus")Integer dataStatus) {
		return projectApplicationService.projectApplication(projectApplicationVO,dataStatus);
	}

	@DeleteMapping(value="/projectApplication/{xmsqId}")
	@ApiOperation(value = "项目申请-删除", notes = "参数：项目申请ID")
	public ResponseWrapper delete(@PathVariable("xmsqId") String xmsqId) {
		return projectApplicationFeign.delete(xmsqId);
	}

	@GetMapping(value="/projectApplication/{xmsqId}")
	@ApiOperation(value = "项目申请-查询详情", notes = "参数：项目申请ID")
	public ResponseWrapper getProjectApplicationInfoById(@PathVariable("xmsqId") String xmsqId) {
		return projectApplicationFeign.getProjectApplicationInfoById(xmsqId);
	}

}
