package com.unis.project.app.controller;

import com.unis.common.utils.ResponseWrapper;
import com.unis.project.app.service.ProjectService;
import com.unis.project.common.vo.ProjectQueryVO;
import com.unis.project.common.vo.ProjectVO;
import com.unis.project.client.ProjectFeign;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @author WangZL
 * @version 1.0
 * @Title ProjectController
 * @ProjectName 项目信息 ProjectController
 * @Description  TODO
 * @date 2019/07/22 17点26分
 */
@RestController
@Api(value = "项目信息 ", tags = "项目信息")
@Slf4j
public class ProjectController {

	@Autowired
	private ProjectFeign projectFeign;

	@Autowired
	private ProjectService projectService;

	@ApiOperation(value = "项目信息-查询列表", notes = "参数：项目方案标题，项目年度")
	@PostMapping(value = "/projectList")
	public ResponseWrapper getProjectList(@RequestBody ProjectQueryVO projectQueryVO){
		return projectFeign.getProjectList(projectQueryVO);
	}

	@ApiOperation(value = "项目信息-分页查询列表", notes = "参数：项目方案标题，项目年度")
	@PostMapping(value = "/projectPage/{pageNum}/{pageSize}")
	public ResponseWrapper getProjectPage(@RequestBody ProjectQueryVO projectQueryVO ,@PathVariable("pageNum")Integer pageNum,
										  @PathVariable("pageSize") Integer pageSize) {
		return projectFeign.getProjectPage(projectQueryVO,pageNum,pageSize);
	}

	@PostMapping(value = "/project/{dataStatus}")
	@ApiOperation(value = "项目信息-根据状态保存数据", notes = "参数：项目信息，操作状态 0、保存。1、发布")
	public ResponseWrapper project(@RequestBody ProjectVO projectVO,@PathVariable("dataStatus")Integer dataStatus) {
		return projectService.project(projectVO,dataStatus);
	}

	@DeleteMapping(value="/project/{xmId}")
	@ApiOperation(value = "项目信息-删除", notes = "参数：项目信息ID")
	public ResponseWrapper delete(@PathVariable("xmId") String xmId) {
		return projectFeign.delete(xmId);
	}

	@GetMapping(value="/project/{xmId}")
	@ApiOperation(value = "项目信息-查询详情", notes = "参数：项目信息ID")
	public ResponseWrapper getProjectInfoById(@PathVariable("xmId") String xmId) {
		return projectFeign.getProjectInfoById(xmId);
	}

	@PostMapping(value="/projectInfo")
	@ApiOperation(value = "项目信息-获取已发布最新项目信息", notes = "参数：")
	public ResponseWrapper getProjectInfo() {
		return projectFeign.getProjectInfo();
	}

	@PostMapping(value="/updateReleaseStatus/{xmId}")
	@ApiOperation(value = "项目信息-发布", notes = "参数：项目信息ID")
	public ResponseWrapper updateReleaseStatus(@PathVariable("xmId") String xmId) {
		return projectFeign.updateReleaseStatus(xmId);
	}
	
}
