package com.unis.project.app.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.unis.common.utils.CurrentUtil;
import com.unis.common.utils.JsonUtils;
import com.unis.common.utils.ResponseWrapper;
import com.unis.project.app.service.ProjectApprovalAcceptanceService;
import com.unis.project.client.ProjectApplicationFeign;
import com.unis.project.client.ProjectApprovalAcceptanceFeign;
import com.unis.project.client.UserClient;
import com.unis.project.common.constant.BaseStatusConstant;
import com.unis.project.common.constant.ProjectStatusConstant;
import com.unis.project.common.vo.ProjectApplicationVO;
import com.unis.project.common.vo.ProjectApprovalAcceptanceVO;
import com.unis.project.common.vo.backlog.ReceiverCollectionVO;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.util.Arrays;
import java.util.Date;
import java.util.List;


/**
 * @author WangZL
 * @version 1.0
 * @Title ProjectApprovalAcceptanceServiceImpl
 * @ProjectName ProjectApprovalAcceptanceServiceImpl
 * @Description  TODO
 * @date 2019/07/22 17点26分
 */
@Service
public class ProjectApprovalAcceptanceServiceImpl implements ProjectApprovalAcceptanceService {

	@Autowired
	private ProjectApprovalAcceptanceFeign projectApprovalAcceptanceFeign;
	@Autowired
	private UserClient userClient;
	@Autowired
	private ProjectApplicationFeign projectApplicationFeign;

	private static final Integer PROJECT_APPROVAL = 1;//项目立项类型：立项审批
	private static final Integer PROJECT_ACCEPTANCE = 2;//项目立项类型：项目验收

	private static final Integer APPROVAL_REJECT = 0;//审批状态：驳回
	private static final Integer APPROVAL_PASSED = 1;//审批状态：通过

	/**
	 * 根据状态保存数据
	 * @param projectApprovalAcceptanceVO
	 * @param dataStatus
	 * @return
	 */
	@Override
	public ResponseWrapper projectApprovalAcceptance(ProjectApprovalAcceptanceVO projectApprovalAcceptanceVO, Integer dataStatus) {
		ResponseWrapper responseWrapper = null;
		ResponseWrapper projectApplicationWrapper = null;
		if (projectApprovalAcceptanceVO != null){
			String xmsqId = projectApprovalAcceptanceVO.getXmsqId();
			Integer xmlxlx = projectApprovalAcceptanceVO.getXmlxlx();
			if (StringUtils.isEmpty(xmsqId)){
				return ResponseWrapper.error("缺少必传参数，请联系管理员！");
			}
			projectApplicationWrapper = projectApplicationFeign.getProjectApplicationInfoById(xmsqId);
			if (projectApplicationWrapper.getInfoCode() != 200) {
				return projectApplicationWrapper;
			}
			ProjectApplicationVO projectApplicationVO = JsonUtils.parse(JsonUtils.serialize(projectApplicationWrapper.getData()), ProjectApplicationVO.class);
			Integer xmlxzt = projectApplicationVO.getXmlxzt();
			if (xmlxzt.equals(ProjectStatusConstant.APPLICATION.getValue())
					&& xmlxlx.equals(PROJECT_APPROVAL)) {//判断上级状态是否是已申请,并且立项类型是立项审批
				responseWrapper = this.projectApprovalAcceptanceinfo(projectApprovalAcceptanceVO,projectApplicationVO,dataStatus,xmlxlx);
			} else if (xmlxzt.equals(ProjectStatusConstant.PROJECT_PROGRESS.getValue())
					&& xmlxlx.equals(PROJECT_ACCEPTANCE)){//判断上级状态是否是项目进度,并且立项类型是项目验收
				responseWrapper = this.projectApprovalAcceptanceinfo(projectApprovalAcceptanceVO,projectApplicationVO,dataStatus,xmlxlx);
			} else {
				return ResponseWrapper.error("操作流程错误，请联系管理员");
			}
		} else {
			return ResponseWrapper.error("操作失误，请联系管理员");
		}
		return responseWrapper;
	}

	/**
	 * 根据状态审批
	 * @param projectApprovalAcceptanceVO
	 * @return
	 */
	@Override
	public ResponseWrapper projectApprovalAcceptanceApproval(ProjectApprovalAcceptanceVO projectApprovalAcceptanceVO) {
		ResponseWrapper responseWrapper = null;
		ResponseWrapper projectApplicationWrapper = null;
		if (projectApprovalAcceptanceVO != null){
			String xmsqId = projectApprovalAcceptanceVO.getXmsqId();
			Integer xmlxlx = projectApprovalAcceptanceVO.getXmlxlx();
			if (StringUtils.isEmpty(xmsqId)){
				return ResponseWrapper.error("缺少必传参数，请联系管理员！");
			}
			projectApplicationWrapper = projectApplicationFeign.getProjectApplicationInfoById(xmsqId);
			if (projectApplicationWrapper.getInfoCode() != 200) {
				return projectApplicationWrapper;
			}
			ProjectApplicationVO projectApplicationVO = JsonUtils.parse(JsonUtils.serialize(projectApplicationWrapper.getData()), ProjectApplicationVO.class);
			Integer xmlxzt = projectApplicationVO.getXmlxzt();
			if (xmlxzt.equals(ProjectStatusConstant.ACCEPTANCE_APPLICATION.getValue())
					&& xmlxlx.equals(PROJECT_ACCEPTANCE)){//判断上级状态是否是验收申请,并且立项类型是项目验收
				Integer psjgzt = projectApprovalAcceptanceVO.getPsjgzt();
				if (psjgzt.equals(APPROVAL_REJECT)){//驳回，更新状态为项目进度
					projectApplicationVO.setXmlxzt(ProjectStatusConstant.PROJECT_PROGRESS.getValue());
				} else if (psjgzt.equals(APPROVAL_PASSED)){//通过，更新状态为已验收
					projectApplicationVO.setXmlxzt(ProjectStatusConstant.ACCEPTANCE.getValue());
				} else {
					return ResponseWrapper.error("操作流程错误，请联系管理员");
				}
				projectApplicationVO.setRefreshDate(new Date());
				projectApplicationVO.setRefresher(CurrentUtil.getUserName());
				projectApplicationVO.setRefresherCode(CurrentUtil.getUserId());
				projectApplicationWrapper = projectApplicationFeign.update(projectApplicationVO);
				if (projectApplicationWrapper.getInfoCode() != 200) {
					return projectApplicationWrapper;
				}
				projectApprovalAcceptanceVO.setRefresher(CurrentUtil.getUserName());
				projectApprovalAcceptanceVO.setRefresherCode(CurrentUtil.getUserId());
				projectApprovalAcceptanceVO.setOrgName(CurrentUtil.getOrgName());
				responseWrapper = projectApprovalAcceptanceFeign.update(projectApprovalAcceptanceVO);
			} else {
				return ResponseWrapper.error("操作流程错误，请联系管理员");
			}
		} else {
			return ResponseWrapper.error("操作失误，请联系管理员");
		}
		return responseWrapper;
	}

	/**
	 * 根据项目立项类型保存数据
	 * @param projectApprovalAcceptanceVO 项目立项验信息
	 * @param projectApplicationVO 项目申请信息
	 * @param dataStatus 操作状态 0、保存。1、提交
	 * @param xmlxlx 项目立项类型：1、立项审批。2、项目验收
	 * @return
	 */
	public ResponseWrapper projectApprovalAcceptanceinfo(ProjectApprovalAcceptanceVO projectApprovalAcceptanceVO,
													 ProjectApplicationVO projectApplicationVO,Integer dataStatus,Integer xmlxlx) {
		ResponseWrapper responseWrapper = null;
		ResponseWrapper projectApplicationWrapper = null;
		if (dataStatus.equals(BaseStatusConstant.STATUS_SAVE)) {//保存
			if (StringUtils.isEmpty(projectApprovalAcceptanceVO.getXmlxysId())) {
				projectApprovalAcceptanceVO.setCreateor(CurrentUtil.getUserName());
				responseWrapper = projectApprovalAcceptanceFeign.insert(projectApprovalAcceptanceVO);
			} else {
				projectApprovalAcceptanceVO.setRefresher(CurrentUtil.getUserName());
				responseWrapper = projectApprovalAcceptanceFeign.update(projectApprovalAcceptanceVO);
			}
		} else if (dataStatus.equals(BaseStatusConstant.STATUS_SUBMIT)) {//提交
			if (xmlxlx.equals(PROJECT_APPROVAL)) {//立项类型是立项审批
				projectApplicationVO.setXmlxzt(ProjectStatusConstant.PROJECT_APPROVAL.getValue());//更新立项状态为立项审批
			} else if (xmlxlx.equals(PROJECT_ACCEPTANCE)) {//立项类型是项目验收
				projectApplicationVO.setXmlxzt(ProjectStatusConstant.ACCEPTANCE_APPLICATION.getValue());//更新立项状态为验收申请
			}
			//获取待办人员
			ResponseWrapper result = userClient.getUserByRole(Arrays.asList(new String[]{"R0000158"}));
			Object resultObj = result.getData();
			List<ReceiverCollectionVO> resultList = Lists.newArrayList();
			if (result.isSuccess() && !ObjectUtils.isEmpty(resultObj)) {
				JSONArray objArr = JSON.parseArray(JsonUtils.serialize(resultObj));
				if (!ObjectUtils.isEmpty(objArr)) {
					objArr.forEach(jsonObj -> {
						JSONObject obj = (JSONObject) jsonObj;
						ReceiverCollectionVO receiverCollectionVO = new ReceiverCollectionVO();
						receiverCollectionVO.setUserId(obj.getString("userCode"));
						receiverCollectionVO.setUserName(obj.getString("userName"));
						resultList.add(receiverCollectionVO);
					});
				}
			}
			projectApprovalAcceptanceVO.setCompanyName(CurrentUtil.getOrgName());
			projectApprovalAcceptanceVO.setSubmitter(CurrentUtil.getUserName());
			projectApprovalAcceptanceVO.setSubmitterId(CurrentUtil.getUserId());
			projectApprovalAcceptanceVO.setUserList(resultList);
			projectApplicationVO.setRefreshDate(new Date());
			projectApplicationVO.setRefresher(CurrentUtil.getUserName());
			projectApplicationWrapper = projectApplicationFeign.update(projectApplicationVO);
			if (projectApplicationWrapper.getInfoCode() != 200){
				return projectApplicationWrapper;
			}
			if (StringUtils.isEmpty(projectApprovalAcceptanceVO.getXmlxysId())) {
				projectApprovalAcceptanceVO.setCreateor(CurrentUtil.getUserName());
				responseWrapper = projectApprovalAcceptanceFeign.insert(projectApprovalAcceptanceVO);
			} else {
				projectApprovalAcceptanceVO.setRefresher(CurrentUtil.getUserName());
				responseWrapper = projectApprovalAcceptanceFeign.update(projectApprovalAcceptanceVO);
			}
		} else {
			return ResponseWrapper.error("操作失误，请联系管理员");
		}
		return responseWrapper;
	}
}
