package com.unis.project.app.service.impl;

import com.unis.common.utils.CurrentUtil;
import com.unis.common.utils.JsonUtils;
import com.unis.common.utils.ResponseWrapper;
import com.unis.project.app.service.ProjectBiddingService;
import com.unis.project.client.ProjectApplicationFeign;
import com.unis.project.client.ProjectBiddingFeign;
import com.unis.project.common.constant.BaseStatusConstant;
import com.unis.project.common.constant.ProjectStatusConstant;
import com.unis.project.common.vo.ProjectApplicationVO;
import com.unis.project.common.vo.ProjectBidingVO;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 * @Author zoujieli
 * @Description
 * @Date 2019/10/28 15:11
 * Version 1.0
 */
@Service
public class ProjectBiddingServiceImpl implements ProjectBiddingService {

    private static final Integer DESIGN_BIDDING = 1;//项目招标类型：设计招标
    private static final Integer SUPERVISION_BIDDING = 2;//项目招标类型：监理招标
    private static final Integer CONSTRUCTION_BIDDING = 3;//项目招标类型：建设招标

    @Autowired
    private ProjectBiddingFeign projectBiddingFeign;

    @Autowired
    private ProjectApplicationFeign projectApplicationFeign;

    /**
     * 根据状态保存数据
     * @param projectBidingVO
     * @param dataStatus
     * @return
     */
    @Override
    public ResponseWrapper insertProjectBidding(ProjectBidingVO projectBidingVO, Integer dataStatus) {
        ResponseWrapper responseWrapper = null;
        ResponseWrapper projectApplicationWrapper = null;
        if (projectBidingVO != null){
            String xmsqId = projectBidingVO.getXmsqId();
            Integer xmzblx = projectBidingVO.getXmzblx();
            if (StringUtils.isEmpty(xmsqId)){
                return ResponseWrapper.error("缺少必传参数，请联系管理员！");
            }
            projectApplicationWrapper = projectApplicationFeign.getProjectApplicationInfoById(xmsqId);
            if (projectApplicationWrapper.getInfoCode() != 200) {
                return projectApplicationWrapper;
            }
            ProjectApplicationVO projectApplicationVO = JsonUtils.parse(JsonUtils.serialize(projectApplicationWrapper.getData()), ProjectApplicationVO.class);
            Integer xmlxzt = projectApplicationVO.getXmlxzt();
            if ((xmlxzt.equals(ProjectStatusConstant.APPLICATION.getValue()) ||
                    xmlxzt.equals(ProjectStatusConstant.PROJECT_APPROVAL.getValue()) ||
                    xmlxzt.equals(ProjectStatusConstant.SUPERVISION_BIDDING.getValue()) ||
                    xmlxzt.equals(ProjectStatusConstant.CONSTRUCTION_BIDDING.getValue()))
                    && xmzblx.equals(DESIGN_BIDDING)) {//判断上级状态为已申请、立项审批、监理招标、建设招标都可以,并且招标类型是设计招标
                responseWrapper = this.projectBiddinginfo(projectBidingVO,projectApplicationVO,dataStatus,xmzblx);
            } else if (xmlxzt.equals(ProjectStatusConstant.PROJECT_APPROVAL.getValue())
                    && xmzblx.equals(SUPERVISION_BIDDING)){//判断上级状态是否是立项审批,并且招标类型是设计招标
                responseWrapper = this.projectBiddinginfo(projectBidingVO,projectApplicationVO,dataStatus,xmzblx);
            } else if (xmlxzt.equals(ProjectStatusConstant.SUPERVISION_BIDDING.getValue())
                    && xmzblx.equals(CONSTRUCTION_BIDDING)){//判断上级状态是否是监理招标,并且招标类型是设计招标
                responseWrapper = this.projectBiddinginfo(projectBidingVO,projectApplicationVO,dataStatus,xmzblx);
            } else {
                return ResponseWrapper.error("操作流程错误，请联系管理员");
            }
        } else {
            return ResponseWrapper.error("操作失误，请联系管理员");
        }
        return responseWrapper;
    }

    /**
     * 根据项目立项类型保存数据
     * @param projectBidingVO 项目招标信息
     * @param projectApplicationVO 项目申请信息
     * @param dataStatus 操作状态 0、保存。1、提交
     * @param xmzblx 项目招标类型：1、设计招标。2、监理招标。3、建设招标
     * @return
     */
    public ResponseWrapper projectBiddinginfo(ProjectBidingVO projectBidingVO, ProjectApplicationVO projectApplicationVO,Integer dataStatus,Integer xmzblx) {
        ResponseWrapper responseWrapper = null;
        ResponseWrapper projectApplicationWrapper = null;
        if (dataStatus.equals(BaseStatusConstant.STATUS_SAVE)) {//保存
            if (StringUtils.isEmpty(projectBidingVO.getXmzbId())) {
                projectBidingVO.setCreateor(CurrentUtil.getUserName());
                responseWrapper = projectBiddingFeign.insertProjectBidding(projectBidingVO);
            } else {
                projectBidingVO.setRefresher(CurrentUtil.getUserName());
                responseWrapper = projectBiddingFeign.updateProjectBidding(projectBidingVO);
            }
        } else if (dataStatus.equals(BaseStatusConstant.STATUS_SUBMIT)) {//提交
            if (xmzblx.equals(DESIGN_BIDDING)) {//招标类型是设计招标
                projectApplicationVO.setXmzbztDate(new Date());
                projectApplicationVO.setXmzbzt(ProjectStatusConstant.DESIGN_BIDDING.getValue());//更新招标状态为设计招标
            } else if (xmzblx.equals(SUPERVISION_BIDDING)) {//招标类型是监理招标
                projectApplicationVO.setRefreshDate(new Date());
                projectApplicationVO.setXmlxzt(ProjectStatusConstant.SUPERVISION_BIDDING.getValue());//更新立项状态为监理招标
            } else if (xmzblx.equals(CONSTRUCTION_BIDDING)) {//招标类型是建设招标
                projectApplicationVO.setRefreshDate(new Date());
                projectApplicationVO.setXmlxzt(ProjectStatusConstant.CONSTRUCTION_BIDDING.getValue());//更新立项状态为建设招标
            }
            projectApplicationVO.setRefresher(CurrentUtil.getUserName());
            projectApplicationWrapper = projectApplicationFeign.update(projectApplicationVO);
            if (projectApplicationWrapper.getInfoCode() != 200){
                return projectApplicationWrapper;
            }
            if (StringUtils.isEmpty(projectBidingVO.getXmzbId())) {
                projectBidingVO.setCreateor(CurrentUtil.getUserName());
                responseWrapper = projectBiddingFeign.insertProjectBidding(projectBidingVO);
            } else {
                projectBidingVO.setRefresher(CurrentUtil.getUserName());
                responseWrapper = projectBiddingFeign.updateProjectBidding(projectBidingVO);
            }
        } else {
            return ResponseWrapper.error("操作失误，请联系管理员");
        }
        return responseWrapper;
    }
}

