package com.unis.project.app.service.impl;

import com.unis.common.utils.CurrentUtil;
import com.unis.common.utils.DateUtil;
import com.unis.common.utils.JsonUtils;
import com.unis.common.utils.ResponseWrapper;
import com.unis.project.app.service.ProjectFundApplicationService;
import com.unis.project.client.ProjectApplicationFeign;
import com.unis.project.client.ProjectFundApplicationFeign;
import com.unis.project.client.ProjectMonthlyProgressReportFeign;
import com.unis.project.client.ProjectProgressBriefingFeign;
import com.unis.project.common.constant.BaseStatusConstant;
import com.unis.project.common.constant.ProjectStatusConstant;
import com.unis.project.common.vo.*;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * @Author zoujieli
 * @Description
 * @Date 2019/10/28 15:11
 * Version 1.0
 */
@Service
public class ProjectFundApplicationServiceImpl implements ProjectFundApplicationService {

    private static final Integer APPROVAL_REJECT = 0;//审批状态：驳回
    private static final Integer APPROVAL_PASSED = 1;//审批状态：通过

    private static final Integer TO_BE_PEPORTED = 1;//项目进度状态：待上报
    private static final Integer PEPORTED = 2;//项目进度状态：已上报
    private static final Integer NOT_PEPORTED = 3;//项目进度状态：未上报

    @Autowired
    private ProjectFundApplicationFeign projectFundApplicationFeign;

    @Autowired
    private ProjectApplicationFeign projectApplicationFeign;

    @Autowired
    private ProjectMonthlyProgressReportFeign projectMonthlyProgressReportFeign;

    @Autowired
    private ProjectProgressBriefingFeign projectProgressBriefingFeign;

    /**
     * 根据状态保存数据
     * @param projectFundApplicationVOList
     * @param dataStatus
     * @return
     */
    @Override
    public ResponseWrapper insertProjectFundApplication(List<ProjectFundApplicationVO> projectFundApplicationVOList, Integer dataStatus) {
        ResponseWrapper responseWrapper = null;
        ResponseWrapper projectApplicationWrapper = null;
        if (projectFundApplicationVOList != null && projectFundApplicationVOList.size() > 0){
            String xmsqId = projectFundApplicationVOList.get(0).getXmsqId();
            if (StringUtils.isEmpty(xmsqId)){
                return ResponseWrapper.error("缺少必传参数，请联系管理员！");
            }
            projectApplicationWrapper = projectApplicationFeign.getProjectApplicationInfoById(xmsqId);
            if (projectApplicationWrapper.getInfoCode() != 200) {
                return projectApplicationWrapper;
            }
            ProjectApplicationVO projectApplicationVO = JsonUtils.parse(JsonUtils.serialize(projectApplicationWrapper.getData()), ProjectApplicationVO.class);
            Integer xmlxzt = projectApplicationVO.getXmlxzt();
            Integer xmzbzt = projectApplicationVO.getXmzbzt();
            if (xmlxzt.equals(ProjectStatusConstant.CONSTRUCTION_BIDDING.getValue())
                    && xmzbzt.equals(ProjectStatusConstant.REPORT_APPROVED.getValue())) {//判断上级招标状态是否是可研报告已批复,并且立项状态是建设招标
                if (dataStatus.equals(BaseStatusConstant.STATUS_SAVE)) {//保存

                } else if (dataStatus.equals(BaseStatusConstant.STATUS_SUBMIT)) {//提交
                    projectApplicationVO.setXmlxzt(ProjectStatusConstant.FUND_APPLICATION.getValue());//更新立项状态为资金申请
                    projectApplicationVO.setRefreshDate(new Date());
                    projectApplicationVO.setRefresher(CurrentUtil.getUserName());
                    projectApplicationVO.setRefresherCode(CurrentUtil.getUserId());
                    projectApplicationWrapper = projectApplicationFeign.update(projectApplicationVO);
                    if (projectApplicationWrapper.getInfoCode() != 200){
                        return projectApplicationWrapper;
                    }
                    //当资金申请提交后，生成一条本月进度简报数据
                    ProjectMonthlyProgressReportQueryVO projectMonthlyProgressReportQueryVO = new ProjectMonthlyProgressReportQueryVO();
                    projectMonthlyProgressReportQueryVO.setJdydYear(DateUtil.getYearString());
                    projectMonthlyProgressReportQueryVO.setJdydMonth(DateUtil.getMonthString());
                    //获取当月进度月报数据
                    ResponseWrapper reportWrapper = projectMonthlyProgressReportFeign.getProjectMonthlyProgressReportList(projectMonthlyProgressReportQueryVO);
                    if (reportWrapper.getInfoCode() != 200){
                        return reportWrapper;
                    }
                    List<ProjectMonthlyProgressReportVO> projectMonthlyProgressReportVOList = JsonUtils.parseList(JsonUtils.serialize(reportWrapper.getData()),ProjectMonthlyProgressReportVO.class);
                    if (projectMonthlyProgressReportVOList != null && projectMonthlyProgressReportVOList.size() > 0){
                        String xmjdybId = projectMonthlyProgressReportVOList.get(0).getXmjdybId();
                        ProjectProgressBriefingVO projectProgressBriefingVO = new ProjectProgressBriefingVO();
                        projectProgressBriefingVO.setXmjdybId(xmjdybId);
                        projectProgressBriefingVO.setXmsqId(xmsqId);
                        //截止时间大于等于今天状态为待上报，否则为未上报
                        if (this.comparisonTime(projectMonthlyProgressReportVOList.get(0).getYbjzrq())){
                            projectProgressBriefingVO.setXmjdzt(TO_BE_PEPORTED);
                        } else {
                            projectProgressBriefingVO.setXmjdzt(NOT_PEPORTED);
                        }
                        projectProgressBriefingVO.setCreateor(CurrentUtil.getUserName());
                        projectProgressBriefingVO.setCreateorCode(CurrentUtil.getUserId());
                        //保存一条本月进度简报数据
                        ResponseWrapper saveWrapper = projectProgressBriefingFeign.insert(projectProgressBriefingVO);
                        if (saveWrapper.getInfoCode() != 200){
                            return saveWrapper;
                        }
                    }
                } else {
                    return ResponseWrapper.error("操作失误，请联系管理员");
                }
                responseWrapper = projectFundApplicationFeign.updateProjectFundApplicationList(projectFundApplicationVOList);
            } else {
                return ResponseWrapper.error("操作流程错误，请联系管理员");
            }
        } else {
            return ResponseWrapper.error("操作失误，请联系管理员");
        }
        return responseWrapper;
    }

    /**
     * 审批
     * @param projectFundApplicationVO
     * @return
     */
    @Override
    public ResponseWrapper projectFundApplicationApproval(ProjectFundApplicationVO projectFundApplicationVO) {
        ResponseWrapper responseWrapper = null;
        ResponseWrapper projectApplicationWrapper = null;
        /*if (projectFundApplicationVO != null){
            String xmsqId = projectFundApplicationVO.getXmsqId();
            if (StringUtils.isEmpty(xmsqId)){
                return ResponseWrapper.error("缺少必传参数，请联系管理员！");
            }
            projectApplicationWrapper = projectApplicationFeign.getProjectApplicationInfoById(xmsqId);
            if (projectApplicationWrapper.getInfoCode() != 200) {
                return projectApplicationWrapper;
            }
            ProjectApplicationVO projectApplicationVO = JsonUtils.parse(JsonUtils.serialize(projectApplicationWrapper.getData()), ProjectApplicationVO.class);
            Integer xmlxzt = projectApplicationVO.getXmlxzt();
            if (xmlxzt.equals(ProjectStatusConstant.FUND_APPLICATION.getValue())){//判断上级状态是否是资金申请
                Integer psjgzt = projectFundApplicationVO.getPsjgzt();
                if (psjgzt.equals(APPROVAL_REJECT)){//驳回
                    projectApplicationVO.setXmlxzt(ProjectStatusConstant.CONSTRUCTION_BIDDING.getValue());//更新 立项状态为建设招标
                    projectApplicationVO.setXmzbzt(ProjectStatusConstant.REPORT_APPROVED.getValue());//更新 招标状态为可研报告已批复
                } else if (psjgzt.equals(APPROVAL_PASSED)){//通过，更新状态为资金批复
                    projectApplicationVO.setXmlxzt(ProjectStatusConstant.FUND_APPROVAL.getValue());
                } else {
                    return ResponseWrapper.error("操作流程错误，请联系管理员");
                }
                projectApplicationVO.setRefreshDate(new Date());
                projectApplicationVO.setRefresher(CurrentUtil.getUserName());
                projectApplicationVO.setRefresherCode(CurrentUtil.getUserId());
                projectApplicationWrapper = projectApplicationFeign.update(projectApplicationVO);
                if (projectApplicationWrapper.getInfoCode() != 200) {
                    return projectApplicationWrapper;
                }
                projectFundApplicationVO.setRefreshDate(new Date());
                projectFundApplicationVO.setRefresher(CurrentUtil.getUserName());
                projectFundApplicationVO.setRefresherCode(CurrentUtil.getUserId());
                responseWrapper = projectFundApplicationFeign.update(projectFundApplicationVO);
            } else {
                return ResponseWrapper.error("操作流程错误，请联系管理员");
            }
        } else {
            return ResponseWrapper.error("操作失误，请联系管理员");
        }*/
        return responseWrapper;
    }

    /**
     * 判断截止时间和当前时间
     * @param deadline
     * @return
     */
    public boolean comparisonTime(Date deadline){
        DateFormat df = new SimpleDateFormat("yyyy-MM-dd");
        Date currentTime = null;
        Date deadlineDate = null;
        try {
            currentTime = df.parse(df.format(new Date()));
            deadlineDate = df.parse(df.format(deadline));
        } catch (ParseException e) {

        }
        if (deadlineDate.getTime() >= currentTime.getTime()) {
            return true;
        } else {
            return false;
        }
    }
}

