package com.unis.project.app.service.impl;

import com.unis.common.utils.CurrentUtil;
import com.unis.common.utils.JsonUtils;
import com.unis.common.utils.ResponseWrapper;
import com.unis.project.app.service.ProjectMonthlyProgressReportService;
import com.unis.project.client.ProjectApplicationFeign;
import com.unis.project.client.ProjectMonthlyProgressReportFeign;
import com.unis.project.client.ProjectProgressBriefingFeign;
import com.unis.project.common.vo.ProjectMonthlyProgressReportVO;
import com.unis.project.common.vo.ProjectProgressBriefingVO;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * @author WangZL
 * @version 1.0
 * @Title ProjectMonthlyProgressReportServiceImpl
 * @ProjectName ProjectMonthlyProgressReportServiceImpl
 * @Description  TODO
 * @date 2019/07/22 17点26分
 */
@Service
public class ProjectMonthlyProgressReportServiceImpl implements ProjectMonthlyProgressReportService {

	@Autowired
	private ProjectMonthlyProgressReportFeign projectMonthlyProgressReportFeign;

	@Autowired
	private ProjectProgressBriefingFeign projectProgressBriefingFeign;

	@Autowired
	private ProjectApplicationFeign projectApplicationFeign;

	private static final Integer TO_BE_PEPORTED = 1;//项目进度状态：待上报
	private static final Integer NOT_PEPORTED = 3;//项目进度状态：未上报

	/**
	 * 更新项目进度月报数据
	 * @param projectMonthlyProgressReportVO
	 * @return
	 */
	@Override
	public ResponseWrapper updateProjectMonthlyProgressReport(ProjectMonthlyProgressReportVO projectMonthlyProgressReportVO) {
		ResponseWrapper responseWrapper = null;
		String xmjdybId = projectMonthlyProgressReportVO.getXmjdybId();
		Date ybjzrq = projectMonthlyProgressReportVO.getYbjzrq();
		if (ybjzrq == null){
			return ResponseWrapper.success(HttpStatus.BAD_REQUEST.value(), "有必传参数为空", null);
		} else if (StringUtils.isEmpty(xmjdybId)) {
			return ResponseWrapper.success(HttpStatus.BAD_REQUEST.value(), "有必传参数为空", null);
		}
		responseWrapper = projectMonthlyProgressReportFeign.update(projectMonthlyProgressReportVO);
		if (responseWrapper.getInfoCode() != 200) {
			return responseWrapper;
		}
		//当选择截止日期大于今天（含今天）改变 对应简报状态
		if (this.comparisonTime(ybjzrq)){
			//查询月度ID 并且状态为未上报数据
			ResponseWrapper getWrapper = projectProgressBriefingFeign.getProjectMaterialDetailsListByxmjdzt(xmjdybId,NOT_PEPORTED);
			if (getWrapper.getInfoCode() != 200) {
				return getWrapper;
			}
			List<ProjectProgressBriefingVO> projectProgressBriefingVOList = JsonUtils.parseList(JsonUtils.serialize(getWrapper.getData()),ProjectProgressBriefingVO.class);
			if (projectProgressBriefingVOList != null && projectProgressBriefingVOList.size() > 0){
				for (ProjectProgressBriefingVO projectProgressBriefingVO : projectProgressBriefingVOList) {
					projectProgressBriefingVO.setXmjdzt(TO_BE_PEPORTED);//项目进度状态改为待上报
					projectProgressBriefingVO.setRefresher(CurrentUtil.getUserName());
					projectProgressBriefingVO.setRefresherCode(CurrentUtil.getUserId());
					projectProgressBriefingFeign.update(projectProgressBriefingVO);
				}
			}
		}
		responseWrapper.setData("操作完成");
		return responseWrapper;
	}

	/**
	 * 判断截止时间和当前时间
	 * @param deadline
	 * @return
	 */
	public boolean comparisonTime(Date deadline){
		DateFormat df = new SimpleDateFormat("yyyy-MM-dd");
		Date currentTime = null;
		Date deadlineDate = null;
		try {
			currentTime = df.parse(df.format(new Date()));
			deadlineDate = df.parse(df.format(deadline));
		} catch (ParseException e) {

		}
		if (deadlineDate.getTime() >= currentTime.getTime()) {
			return true;
		} else {
			return false;
		}
	}

}
