package com.unis.project.app.service.impl;

import com.unis.common.utils.CurrentUtil;
import com.unis.common.utils.JsonUtils;
import com.unis.common.utils.ResponseWrapper;
import com.unis.project.app.service.ProjectProgressBriefingService;
import com.unis.project.client.ProjectApplicationFeign;
import com.unis.project.client.ProjectProgressBriefingFeign;
import com.unis.project.common.constant.BaseStatusConstant;
import com.unis.project.common.constant.ProjectStatusConstant;
import com.unis.project.common.vo.ProjectApplicationVO;
import com.unis.project.common.vo.ProjectProgressBriefingVO;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * @author WangZL
 * @version 1.0
 * @Title ProjectProgressBriefingServiceImpl
 * @ProjectName ProjectProgressBriefingServiceImpl
 * @Description  TODO
 * @date 2019/07/22 17点26分
 */
@Service
public class ProjectProgressBriefingServiceImpl implements ProjectProgressBriefingService {

	@Autowired
	private ProjectProgressBriefingFeign projectProgressBriefingFeign;

	@Autowired
	private ProjectApplicationFeign projectApplicationFeign;

	private static final Integer TO_BE_PEPORTED = 1;//项目进度状态：待上报
	private static final Integer PEPORTED = 2;//项目进度状态：已上报
	private static final Integer NOT_PEPORTED = 3;//项目进度状态：未上报

	/**
	 * 根据状态保存数据
	 * @param projectProgressBriefingVOList
	 * @param dataStatus
	 * @return
	 */
	@Override
	public ResponseWrapper projectProgressBriefing(List<ProjectProgressBriefingVO> projectProgressBriefingVOList, Integer dataStatus) {
		ResponseWrapper responseWrapper = null;
		ResponseWrapper projectApplicationWrapper = null;
		int sum = 0;
		if (projectProgressBriefingVOList != null && projectProgressBriefingVOList.size() > 0){
			String xmsqId = projectProgressBriefingVOList.get(0).getXmsqId();
			if (StringUtils.isEmpty(xmsqId)){
				return ResponseWrapper.error("缺少必传参数，请联系管理员！");
			}
			projectApplicationWrapper = projectApplicationFeign.getProjectApplicationInfoById(xmsqId);
			if (projectApplicationWrapper.getInfoCode() != 200) {
				return projectApplicationWrapper;
			}
			ProjectApplicationVO projectApplicationVO = JsonUtils.parse(JsonUtils.serialize(projectApplicationWrapper.getData()), ProjectApplicationVO.class);
			Integer xmlxzt = projectApplicationVO.getXmlxzt();
			if (xmlxzt.equals(ProjectStatusConstant.FUND_APPLICATION.getValue()) ||
					xmlxzt.equals(ProjectStatusConstant.PROJECT_PROGRESS.getValue())) {//判断上级立项状态是否是资金申请或项目进度
				for (ProjectProgressBriefingVO projectProgressBriefingVO : projectProgressBriefingVOList) {
					xmsqId = projectProgressBriefingVO.getXmsqId();
					Integer xmjdzt = projectProgressBriefingVO.getXmjdzt();
					if (StringUtils.isEmpty(xmsqId)) {
						return ResponseWrapper.error("缺少必传参数，请联系管理员！");
					}
					if (xmjdzt.equals(TO_BE_PEPORTED)) {//进度为待上报状态更新数据
						projectProgressBriefingVO.setRefreshDate(new Date());
						projectProgressBriefingVO.setRefresher(CurrentUtil.getUserName());
						projectProgressBriefingVO.setRefresherCode(CurrentUtil.getUserId());
						if (dataStatus.equals(BaseStatusConstant.STATUS_SUBMIT)) {//提交改变状态
							projectProgressBriefingVO.setXmjdzt(PEPORTED);//项目进度状态改为已上报
							sum++;
						}
						responseWrapper = projectProgressBriefingFeign.update(projectProgressBriefingVO);
					} else {
						if (dataStatus.equals(BaseStatusConstant.STATUS_SUBMIT)) {//提交改变状态
							sum++;
						}
					}
				}
				if (sum > 0) {//存在提交数据更新状态
					projectApplicationVO.setXmlxzt(ProjectStatusConstant.PROJECT_PROGRESS.getValue());//更新立项状态为项目进度
					projectApplicationVO.setRefreshDate(new Date());
					projectApplicationVO.setRefresher(CurrentUtil.getUserName());
					projectApplicationVO.setRefresherCode(CurrentUtil.getUserId());
					projectApplicationWrapper = projectApplicationFeign.update(projectApplicationVO);
					if (projectApplicationWrapper.getInfoCode() != 200){
						return projectApplicationWrapper;
					}
				}
			} else {
				return ResponseWrapper.error("操作流程错误，请联系管理员");
			}
		}
		if (responseWrapper == null) {
			return ResponseWrapper.success("操作完成");
		}
		return responseWrapper;
	}

}
