package com.unis.project.app.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.unis.common.utils.CurrentUtil;
import com.unis.common.utils.JsonUtils;
import com.unis.common.utils.ResponseWrapper;
import com.unis.project.app.service.ProjectProposalReportService;
import com.unis.project.client.ProjectApplicationFeign;
import com.unis.project.client.ProjectProposalReportFeign;
import com.unis.project.client.UserClient;
import com.unis.project.common.constant.BaseStatusConstant;
import com.unis.project.common.constant.ProjectStatusConstant;
import com.unis.project.common.vo.ProjectApplicationVO;
import com.unis.project.common.vo.ProjectProposalReportVO;
import com.unis.project.common.vo.backlog.ReceiverCollectionVO;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.util.Arrays;
import java.util.Date;
import java.util.List;


/**
 * @Author zoujieli
 * @Description
 * @Date 2019/10/28 15:11
 * Version 1.0
 */
@Service
public class ProjectProposalReportServiceImpl implements ProjectProposalReportService {

    private static final Integer PROJECT_PROPOSAL = 1;//项目文本类型：项目建议书
    private static final Integer PROJECT_REPORT = 2;//项目文本类型：可研报告

    private static final Integer APPROVAL_REJECT = 0;//审批状态：驳回
    private static final Integer APPROVAL_PASSED = 1;//审批状态：通过

    @Autowired
    private ProjectProposalReportFeign projectProposalReportFeign;

    @Autowired
    private ProjectApplicationFeign projectApplicationFeign;

    @Autowired
    private UserClient userClient;

    /**
     * 根据状态保存数据
     * @param projectProposalReportVO
     * @param dataStatus
     * @return
     */
    @Override
    public ResponseWrapper insertProjectProposalReport(ProjectProposalReportVO projectProposalReportVO, Integer dataStatus) {
        ResponseWrapper responseWrapper = null;
        ResponseWrapper projectApplicationWrapper = null;
        if (projectProposalReportVO != null){
            String xmsqId = projectProposalReportVO.getXmsqId();
            Integer xmwblx = projectProposalReportVO.getXmwblx();
            if (StringUtils.isEmpty(xmsqId)){
                return ResponseWrapper.error("缺少必传参数，请联系管理员！");
            }
            projectApplicationWrapper = projectApplicationFeign.getProjectApplicationInfoById(xmsqId);
            if (projectApplicationWrapper.getInfoCode() != 200) {
                return projectApplicationWrapper;
            }
            ProjectApplicationVO projectApplicationVO = JsonUtils.parse(JsonUtils.serialize(projectApplicationWrapper.getData()), ProjectApplicationVO.class);
            Integer xmzbzt = projectApplicationVO.getXmzbzt();
            if (xmzbzt.equals(ProjectStatusConstant.DESIGN_BIDDING.getValue())
                    && xmwblx.equals(PROJECT_PROPOSAL)) {//判断上级状态是否是设计招标,并且文本类型是项目建议书
                responseWrapper = this.projectProposalReportinfo(projectProposalReportVO,projectApplicationVO,dataStatus,xmwblx);
            } else if (xmzbzt.equals(ProjectStatusConstant.PROPOSAL_APPROVED.getValue())
                    && xmwblx.equals(PROJECT_REPORT)){//判断上级状态是否是建议书已批复,并且文本类型是可研报告
                responseWrapper = this.projectProposalReportinfo(projectProposalReportVO,projectApplicationVO,dataStatus,xmwblx);
            } else {
                return ResponseWrapper.error("操作流程错误，请联系管理员");
            }
        } else {
            return ResponseWrapper.error("操作失误，请联系管理员");
        }
        return responseWrapper;
    }

    /**
     * 审批
     * @param projectProposalReportVO
     * @return
     */
    @Override
    public ResponseWrapper projectProposalReportApproval(ProjectProposalReportVO projectProposalReportVO) {
        ResponseWrapper responseWrapper = null;
        ResponseWrapper projectApplicationWrapper = null;
        if (projectProposalReportVO != null){
            String xmsqId = projectProposalReportVO.getXmsqId();
            Integer xmwblx = projectProposalReportVO.getXmwblx();
            if (StringUtils.isEmpty(xmsqId)){
                return ResponseWrapper.error("缺少必传参数，请联系管理员！");
            }
            projectApplicationWrapper = projectApplicationFeign.getProjectApplicationInfoById(xmsqId);
            if (projectApplicationWrapper.getInfoCode() != 200) {
                return projectApplicationWrapper;
            }
            ProjectApplicationVO projectApplicationVO = JsonUtils.parse(JsonUtils.serialize(projectApplicationWrapper.getData()), ProjectApplicationVO.class);
            Integer xmzbzt = projectApplicationVO.getXmzbzt();
            if (xmzbzt.equals(ProjectStatusConstant.PROPOSAL_SUBMITTED.getValue())
                    && xmwblx.equals(PROJECT_PROPOSAL)){//判断上级状态是否是建议书已提交,并且文本类型是项目建议书
                responseWrapper = this.projectProposalReportApprovalInfo(projectProposalReportVO,projectApplicationVO,xmwblx);
            } else if (xmzbzt.equals(ProjectStatusConstant.REPORT_SUBMITTED.getValue())
                    && xmwblx.equals(PROJECT_REPORT)){//判断上级状态是否是可研报告已提交,并且文本类型是可研报告
                responseWrapper = this.projectProposalReportApprovalInfo(projectProposalReportVO,projectApplicationVO,xmwblx);
            } else {
                return ResponseWrapper.error("操作流程错误，请联系管理员");
            }
        } else {
            return ResponseWrapper.error("操作失误，请联系管理员");
        }
        return responseWrapper;
    }

    /**
     * 根据项目立项类型保存数据
     * @param projectProposalReportVO 项目建议书报告信息
     * @param projectApplicationVO 项目申请信息
     * @param xmwblx 项目文本类型：1、项目建议书。2、可研报告
     * @return
     */
    public ResponseWrapper projectProposalReportApprovalInfo(ProjectProposalReportVO projectProposalReportVO,
                                                         ProjectApplicationVO projectApplicationVO,Integer xmwblx) {
        ResponseWrapper responseWrapper = null;
        ResponseWrapper projectApplicationWrapper = null;
        Integer psjgzt = projectProposalReportVO.getPsjgzt();
        if (xmwblx.equals(PROJECT_PROPOSAL) && psjgzt.equals(APPROVAL_REJECT)){//文本类型为项目建议书，并驳回
            projectApplicationVO.setXmzbzt(ProjectStatusConstant.DESIGN_BIDDING.getValue());//更新状态为设计招标
        } else if (xmwblx.equals(PROJECT_PROPOSAL) && psjgzt.equals(APPROVAL_PASSED)){//文本类型为项目建议书，并通过
            projectApplicationVO.setXmzbzt(ProjectStatusConstant.PROPOSAL_APPROVED.getValue());//更新状态为建议书已批复
        } else if (xmwblx.equals(PROJECT_REPORT) && psjgzt.equals(APPROVAL_REJECT)){//文本类型为可研报告，并驳回
            projectApplicationVO.setXmzbzt(ProjectStatusConstant.PROPOSAL_APPROVED.getValue());//更新状态为建议书已批复
        } else if (xmwblx.equals(PROJECT_REPORT) && psjgzt.equals(APPROVAL_PASSED)){//文本类型为可研报告，并通过
            projectApplicationVO.setXmzbzt(ProjectStatusConstant.REPORT_APPROVED.getValue());//更新状态为可研报告已批复
        } else {
            return ResponseWrapper.error("操作流程错误，请联系管理员");
        }

        projectApplicationVO.setXmzbztDate(new Date());
        projectApplicationVO.setRefresher(CurrentUtil.getUserName());
        projectApplicationVO.setRefresherCode(CurrentUtil.getUserId());
        projectApplicationWrapper = projectApplicationFeign.update(projectApplicationVO);
        if (projectApplicationWrapper.getInfoCode() != 200) {
            return projectApplicationWrapper;
        }
        projectProposalReportVO.setRefreshDate(new Date());
        projectProposalReportVO.setRefresher(CurrentUtil.getUserName());
        projectProposalReportVO.setRefresherCode(CurrentUtil.getUserId());
        projectProposalReportVO.setOrgName(CurrentUtil.getOrgName());
        responseWrapper = projectProposalReportFeign.updateProjectProposalReport(projectProposalReportVO);
        return responseWrapper;
    }

    /**
     * 根据项目文本类型保存数据
     * @param projectProposalReportVO 项目建议书报告信息
     * @param projectApplicationVO 项目申请信息
     * @param dataStatus 操作状态 0、保存。1、提交
     * @param xmwblx 项目文本类型：1、项目建议书。2、可研报告
     * @return
     */
    public ResponseWrapper projectProposalReportinfo(ProjectProposalReportVO projectProposalReportVO,
                                                         ProjectApplicationVO projectApplicationVO,Integer dataStatus,Integer xmwblx) {
        ResponseWrapper responseWrapper = null;
        ResponseWrapper projectApplicationWrapper = null;
        if (dataStatus.equals(BaseStatusConstant.STATUS_SAVE)) {//保存
            if (StringUtils.isEmpty(projectProposalReportVO.getXmjysbgId())) {
                projectProposalReportVO.setCreateor(CurrentUtil.getUserName());
                responseWrapper = projectProposalReportFeign.insertProjectProposalReport(projectProposalReportVO);
            } else {
                projectProposalReportVO.setRefresher(CurrentUtil.getUserName());
                responseWrapper = projectProposalReportFeign.updateProjectProposalReport(projectProposalReportVO);
            }
        } else if (dataStatus.equals(BaseStatusConstant.STATUS_SUBMIT)) {//提交
            //获取待办人员
            ResponseWrapper result = userClient.getUserByRole(Arrays.asList(new String[]{"R0000158"}));
            Object resultObj = result.getData();
            List<ReceiverCollectionVO> resultList = Lists.newArrayList();
            if (result.isSuccess() && !ObjectUtils.isEmpty(resultObj)) {
                JSONArray objArr = JSON.parseArray(JsonUtils.serialize(resultObj));
                if (!ObjectUtils.isEmpty(objArr)) {
                    objArr.forEach(jsonObj -> {
                        JSONObject obj = (JSONObject) jsonObj;
                        ReceiverCollectionVO receiverCollectionVO = new ReceiverCollectionVO();
                        receiverCollectionVO.setUserId(obj.getString("userCode"));
                        receiverCollectionVO.setUserName(obj.getString("userName"));
                        resultList.add(receiverCollectionVO);
                    });
                }
            }
            projectProposalReportVO.setCompanyName(CurrentUtil.getOrgName());
            projectProposalReportVO.setSubmitter(CurrentUtil.getUserName());
            projectProposalReportVO.setSubmitterId(CurrentUtil.getUserId());
            projectProposalReportVO.setUserList(resultList);
            if (xmwblx.equals(PROJECT_PROPOSAL)) {//文本类型是项目建议书
                projectApplicationVO.setXmzbzt(ProjectStatusConstant.PROPOSAL_SUBMITTED.getValue());//更新招标状态为建议书已提交
            } else if (xmwblx.equals(PROJECT_REPORT)) {//文本类型是可研报告
                projectApplicationVO.setXmzbzt(ProjectStatusConstant.REPORT_SUBMITTED.getValue());//更新招标状态为可研报告已提交
            }
            projectApplicationVO.setXmzbztDate(new Date());
            projectApplicationVO.setRefresher(CurrentUtil.getUserName());
            projectApplicationWrapper = projectApplicationFeign.update(projectApplicationVO);
            if (projectApplicationWrapper.getInfoCode() != 200){
                return projectApplicationWrapper;
            }
            if (StringUtils.isEmpty(projectProposalReportVO.getXmjysbgId())) {
                projectProposalReportVO.setCreateor(CurrentUtil.getUserName());
                responseWrapper = projectProposalReportFeign.insertProjectProposalReport(projectProposalReportVO);
            } else {
                projectProposalReportVO.setRefresher(CurrentUtil.getUserName());
                responseWrapper = projectProposalReportFeign.updateProjectProposalReport(projectProposalReportVO);
            }
        } else {
            return ResponseWrapper.error("操作失误，请联系管理员");
        }
        return responseWrapper;
    }
}

